/* ROS Node which spawns a robot in USARSIM, publishes a topic with sensor data and
 * runs a service for driving the robot.
 *
 * Robotics
 * Jacobs University Bremen
 *
 * Flavia Grosan (f.grosan@jacobs-university.de)
 * Alex Tandrau (a.tandrau@jacobs-university.de)
 */

#include "ros/ros.h"
#include "usarsim/Drive.h"
#include "std_msgs/String.h"

#include <pthread.h>
#include <stdio.h>
#include <iostream>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sstream>

#define SERVER_ADDRESS "192.168.0.191"
#define PORT 3000
#define BUFFER 10000

using namespace std;

struct sockaddr_in c_addr;
int c_sock;
int conX;
string buffered, current_output;

void spawn(int argc, char ** argv) {
	stringstream ss;
	ss << "INIT {ClassName USARBot.ATRV} {Name " << argv[4] << "} {Location " << argv[1] << ", " << argv[2] << ", " << argv[3] << "}\r\n";

	ROS_INFO("Spawning robot");

	ROS_INFO("%s", ss.str().c_str());
	send(c_sock, ss.str().c_str(), strlen(ss.str().c_str()), 0);

	ROS_INFO("Robot spawned");
}

bool drive(usarsim::Drive::Request &req,
		   usarsim::Drive::Response &res) {
	ROS_INFO("Driving robot: left = %lf, right = %lf", req.left, req.right);

	stringstream ss;
	ss << "DRIVE {Left " << req.left << "} {Right " << req.right << "}\r\n";

	ROS_INFO("%s", ss.str().c_str());
	send(c_sock, ss.str().c_str(), strlen(ss.str().c_str()), 0);

	res.result = "Success";

	ROS_INFO("Drive sent");
	return true;
}

int main(int argc, char ** argv)
{
	if (argc != 5) {
		ROS_INFO("Usage: usarsim X Y Z Name");
		return 0;
	}

	c_sock = socket(AF_INET, SOCK_STREAM, 0);
	if (c_sock == -1) {
		ROS_INFO("SOCKET ERROR");
		return 0;
	}

	c_addr.sin_addr.s_addr = inet_addr(SERVER_ADDRESS);
	c_addr.sin_port = htons(PORT);
	c_addr.sin_family = AF_INET;

	conX = connect(c_sock, (struct sockaddr*)&c_addr, sizeof(c_addr));
	if (conX != 0) {
		ROS_INFO("Connection Error!");
		ROS_INFO("* Your USARSIM server needs to be up and running!");
		ROS_INFO("* Make sure you have specified the correct IP address");
		ROS_INFO("* Turn off any firewalls");

		return 0;
	}
	
	spawn(argc, argv);
	
	ros::init(argc, argv, argv[4]);
	ros::NodeHandle n;

	stringstream scanner_service, drive_service;
	scanner_service << argv[4] << "/sensor";
	drive_service << argv[4] << "/drive";
	ros::ServiceServer driver = n.advertiseService(drive_service.str().c_str(), drive);

	ROS_INFO("Ready to drive robot.");

	ros::Publisher scanner_pub = n.advertise<std_msgs::String>(scanner_service.str().c_str(), 1000);

	while(ros::ok()) {
		char send_string[BUFFER];
		for (int j = 0; j < BUFFER; j++) send_string[j] = '\0';
		recv(c_sock, send_string, BUFFER - 1, 0);
	
		std_msgs::String msg;
		stringstream ss;
		ss << send_string;

		msg.data = ss.str();
		ROS_INFO("[Publishing] %s", msg.data.c_str());

		scanner_pub.publish(msg);
		ros::spinOnce();
	}

	return 0;
}
